package meerkat.voting.controller.selector;

/**
 * This class is the SelectionData used to initialize a SimpleListCategoriesSelector.
 * It holds a table of question categories.
 * Every answer in the channel choice phase dictates a category of questions to include in the ballot.
 * This class is simply only the data class of these categories.
 * Specifically the questionSelectionsByAnswer member is this categories table:
 * questionSelectionsByAnswer[questionNumber][answerNumber] is an array of question indices to include in ballot
 * whenever the voter answered answerNumber on question questionNumber in the channel choice phase.
 * Also the sharedDefaults member is a category of question indices to include for EVERY voter,
 * regardless of his channel choice answers
 */
public class SimpleCategoriesData extends SelectionData{

    private int[] sharedDefaults;  // a category of questions to include for every voter
    // the categories
    // first index is the question number, the second is the answer number. the value is an array of question indices in this category
    private int[][][] questionSelectionsByAnswer;
    private int maxQuestionNumber;

    public SimpleCategoriesData() {
        sharedDefaults = new int[0];
        questionSelectionsByAnswer = new int[0][][];
        maxQuestionNumber = -1;
    }

    public void setSharedDefaults(int[] sharedDefaultsIndices) {
        this.sharedDefaults = sharedDefaultsIndices.clone();
        maxQuestionNumber = Math.max(maxQuestionNumber, maxIntInArray(sharedDefaultsIndices));
    }

    public int[] getSharedDefaults() {
        return sharedDefaults;
    }

    public void setItem(int questionNumber, int answerNumber, int[] questionIndices) {
        if (questionNumber >= questionSelectionsByAnswer.length) {
            int[][][] tmp = new int[questionNumber+1][][];
            System.arraycopy(questionSelectionsByAnswer, 0, tmp, 0, questionSelectionsByAnswer.length);
            tmp[questionNumber] = new int[0][];
            questionSelectionsByAnswer = tmp;
        }

        if (answerNumber >= questionSelectionsByAnswer[questionNumber].length) {
            int[][] tmp = new int[answerNumber+1][];
            System.arraycopy(questionSelectionsByAnswer[questionNumber], 0,
                    tmp, 0, questionSelectionsByAnswer[questionNumber].length);
            questionSelectionsByAnswer[questionNumber] = tmp;
        }

        questionSelectionsByAnswer[questionNumber][answerNumber] = questionIndices.clone();
        maxQuestionNumber = Math.max(maxQuestionNumber, maxIntInArray(questionIndices));
    }

    public int[] getItem(int questionNumber, int answerNumber) {
        return questionSelectionsByAnswer[questionNumber][answerNumber];
    }

    public int getMaxQuestionNumber() {
        return maxQuestionNumber;
    }


    private static int maxIntInArray(int[] arr) {
        int m = -1;
        for (int i: arr) {
            m = Math.max(i , m);
        }
        return m;
    }

    public String toString () {
        String s = "SimpleCategoriesData:\n";
        s += "shared : " + arrayToString(sharedDefaults) + "\n";
        s += "map : \n";
        for (int questionNumber = 0; questionNumber < questionSelectionsByAnswer.length; ++questionNumber) {
            for (int answerNumber = 0; answerNumber < questionSelectionsByAnswer[questionNumber].length; ++answerNumber) {
                s += "(" + questionNumber + ", " + answerNumber + ") -> " +
                        arrayToString(questionSelectionsByAnswer[questionNumber][answerNumber]) + "\n";
            }
        }
        return s;
    }

    private String arrayToString (int[] a) {
        if (a.length == 0) {
            return "[]";
        }
        String s = "[" + a[0];
        for (int i = 1; i < a.length; ++i) {
            s += ", " + a[i];
        }
        return s + "]";
    }

}
