package meerkat.voting.controller.selector;

import meerkat.protobuf.Voting.BallotAnswer;
import meerkat.protobuf.Voting.BallotQuestion;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.List;

/**
 * A simple implementation of a QuestionSelector.
 * This implementation simply regards every single answer in the channel choice phase as an identifier of a category
 * Every category is an array of ballot questions.
 * Data of categories is initialized and stored by a SimpleCategoriesData object.
 * After receiving the answers from a channel choice phase, this class simply gathers all the categories
 * chosen and compiles the list of ballot questions to include in the ballot for this voter (a question
 * is included if its index appears in any chosen category, or in the default category shared by all voters)
 */
public class SimpleListCategoriesSelector implements QuestionSelector {
    protected final static Logger logger = LoggerFactory.getLogger(SimpleListCategoriesSelector.class);
    private BallotQuestion[] allBallotQuestions;
    private SimpleCategoriesData selectionData;
    private boolean[] isSelected;


    public SimpleListCategoriesSelector() {
        this.allBallotQuestions = null;
        this.selectionData = null;
    }

    @Override
    public void setAllBallotQuestions(List<BallotQuestion> allBallotQuestions) {
        this.allBallotQuestions = (BallotQuestion[])allBallotQuestions.toArray();
        isSelected = new boolean[this.allBallotQuestions.length];
        assertDataValid();
    }

    @Override
    public void setSelectionData(SelectionData data) {
        if (! (data instanceof SimpleCategoriesData)) {
            String errorMessage = "Initialization of SimpleListCategoriesSelector with wrong object type";
            logger.error(errorMessage);
            throw new RuntimeException(errorMessage);
        }
        this.selectionData = (SimpleCategoriesData)data;
        assertDataValid();
    }

    private void assertDataValid () {
        if (selectionData != null && allBallotQuestions != null) {
            int questionsLength = allBallotQuestions.length;
            int maxQuestionNumber = selectionData.getMaxQuestionNumber();
            if (maxQuestionNumber >= questionsLength) {
                String errorMessage = "Selection data refers to question index " + maxQuestionNumber + " while we have only " + questionsLength + " questions totally";
                logger.error(errorMessage);
                throw new IndexOutOfBoundsException(errorMessage);
            }
        }
    }

    @Override
    public int getChannelIdentifier(List<BallotAnswer> channelChoiceAnswers) {
        return 0;
    }

    @Override
    public List<BallotQuestion> selectQuestionsForVoter(List<BallotAnswer> channelChoiceAnswers) {
        java.util.Arrays.fill(isSelected, false);
        markSelected(selectionData.getSharedDefaults());
        markSelectionsOfVoterChannelChoice ((BallotAnswer[])channelChoiceAnswers.toArray());
        int[] questionIndices = extractSelectedIndices();
        List<BallotQuestion> selectedQuestions = new ArrayList<>();
        for (int questionIndex: questionIndices) {
            selectedQuestions.add(allBallotQuestions[questionIndex]);
        }
        return selectedQuestions;
    }

    private void markSelectionsOfVoterChannelChoice(BallotAnswer[] channelChoiceAnswers) {
        for (int q = 0; q < channelChoiceAnswers.length; ++q) {
            BallotAnswer ballotAnswer = channelChoiceAnswers[q];
            assertAnswerLengthIsOne(ballotAnswer, q);
            assertKeyInSelectionData(q, (int)ballotAnswer.getAnswer(0));
            markSelected(selectionData.getItem(q, (int)ballotAnswer.getAnswer(0)));
        }
    }

    private void assertAnswerLengthIsOne (BallotAnswer ballotAnswer, int questionNumber) {
        if (ballotAnswer.getAnswerCount() != 1) {
            String errorMessage = "SimpleListCategoriesSelector expects a single answer for every channel choice question\n";
            errorMessage += "Answer to question number " + (questionNumber+1) + " is";
            for (long i : ballotAnswer.getAnswerList()) {
                errorMessage += " " + i;
            }
            logger.error(errorMessage);
            throw new IllegalArgumentException(errorMessage);
        }
    }

    private void assertKeyInSelectionData(int questionNumber, int answerNumber) {
        int[] questionListToAdd = selectionData.getItem(questionNumber, answerNumber);
        if (null == questionListToAdd) {
            String errorMessage = "SimpleListCategoriesSelector could not resolve answer number " + answerNumber + " for question number " + questionNumber;
            logger.error(errorMessage);
            throw new IllegalArgumentException(errorMessage);
        }
    }

    private void markSelected (int[] questionIndices) {
        for (int i: questionIndices) {
            isSelected[i] = true;
        }
    }

    private int[] extractSelectedIndices() {
        int nSelected = 0;
        for (boolean b: isSelected) {
            if (b) {
                nSelected++;
            }
        }
        int[] res = new int[nSelected];
        int currIndex = 0;
        for (int questionNumber = 0; questionNumber < isSelected.length; ++questionNumber) {
            if (isSelected[questionNumber]) {
                res[currIndex] = questionNumber;
                currIndex++;
            }
        }
        return res;
    }
}
